{ lib, ... }:
  let
    inherit (lib)
      mkOption
      types
    ;
  in {
    imports = [
      ./git.nix
      ./i18n.nix
      ./partition.nix
      ./software.nix
    ];

    options = {
      valhalla = {
        boot = {
          efiMountPoint = mkOption {
            type = types.str;
            description = "The mountpoint of the efi partition.";
            default = "/boot";
          };

          label = mkOption {
            type = types.str;
            description = "The label of the boot entry.";
            default = "OS";
          };
        };

        setupUser = {
          name = mkOption {
            type = types.str;
            description = "The name of the user used to set up the system.";
            default = "heimdall";
          };

          id = mkOption {
            type = types.int;
            description = "The UID of the user used to set up the system.";
            default = 420;
          };
        };

        timeZone = mkOption {
          type = types.nullOr types.str;
          description = "The time zone of the system.";
          default = null;
        };

        keyMap = mkOption {
          type = types.nullOr types.str;
          description = "The console key map of the system.";
          default = null;
        };

        keyboardLayout = mkOption {
          type = types.nullOr types.str;
          description = "The X11 keyboard layout of the system.";
          default = null;
        };
      };
    };
  }
